/* vga.c - funcoes para acessar os modos da placa Trident Super-VGA */

#include <dos.h>
#include <math.h>
#include <stdlib.h>

#include "vga.h"

char palette [256] [3];

/*-------------------------------------------------------------------------*\
 | inic_modo: Inicializa o modo passado por parametro, os modos aceitaveis |
 | estao descritos no arquivo header "vga.h".                              |
 | Recebe: Um inteiro indicando o modo a ser setado.                       |
 | Retorna: Nada.                                                          |
\*-------------------------------------------------------------------------*/
void inic_modo (int modo)
{
  union REGS reg_entr,
	     reg_saida;

  reg_entr.h.ah = 0;
  reg_entr.h.al = modo;
  int86 (0x10, &reg_entr, &reg_saida);
}

/*-------------------------------------------------------------------------*\
 | setMCGA: Coloca um pixel no modo grafico MCGA (320 x 200 com 256 cores).|
 | Recebe: As coordenadas os inteiros correspondentes as coordenadas do    |
 | video e a cor do pixel.                                                 |
 | Retonra: Nada.                                                          |
\*-------------------------------------------------------------------------*/
void setMCGA (int x, int y, int cor)
{
  unsigned offset;
  char far *tela = MK_FP (0xA000, 0x0000); /* Endereco da placa de video. */

  offset = y * 320 + x;
  tela [offset] = cor;
}

/*-------------------------------------------------------------------------*\
 | lineMCGA: Desenha uma linha no modo de video MCGA.                      |
 | Recebe: As coordenadas de origem, as de destino e a cor da linha.       |
 | Retorna: Nada.                                                          |
\*-------------------------------------------------------------------------*/
void lineMCGA (int x1, int y1, int x2, int y2, int cor)
{
  int dx, dy,
      incr1, incr2,
      x, y,
      xend,
      d;

  dx = abs (x2 - x1);
  dy = abs (y2 - y1);

  d = 2 * dx - dy;

  incr1 = 2 * dy;
  incr2 = 2 * (dy - dx);

  if (x1 > x2) {
    x = x2;
    y = y2;
    xend = x1;
  }
  else {
    x = x1;
    y = y1;
    xend = x2;
  }

  setMCGA (x, y, cor);
  while (x < xend) {
    x++;
    if (d < 0)
      d += incr1;
    else {
      y++;
      d += incr2;
    }
    setMCGA (x, y, cor);
  }
}

/*-------------------------------------------------------------------------*\
 | setSVGAMED: Coloca um pixel no modo grafico SVGAMED (800 x 600 com 256  |
 | cores).                                                                 |
 | Recebe: As coordenadas os inteiros correspondentes as coordenadas do    |
 | video e a cor do pixel.                                                 |
 | Retonra: Nada.                                                          |
/*-------------------------------------------------------------------------*/
void setSVGAMED (int x, int y, int cor)
{
  unsigned long offset;

  static char bank,
	      bank_ant = 0;

  char far *tela = MK_FP (0xA000, 0x0000); /* Endereco da placa de video. */

  offset = (unsigned long) y * 800 + x;
  bank = offset >> 16;
  offset = offset & 0xFFFF;

  if (bank != bank_ant) {   /* Setar o banco de memoria de video. */
    asm cli
    asm Push   AX
    asm Push   DX
    asm Mov    AH, bank
    asm Xor    AH, 2
    asm Mov    DX, 0x3c4
    asm Mov    AL, 0Eh
    asm Out    DX, AX
    asm Pop DX
    asm Pop AX
    asm sti
    bank_ant = bank;
  }

  tela [offset] = cor;
}

/*-------------------------------------------------------------------------*\
 | setSVGAHI: Coloca um pixel no modo grafico SVGAHI (1024 x 768 com 256   |
 | cores).                                                                 |
 | Recebe: As coordenadas os inteiros correspondentes as coordenadas do    |
 | video e a cor do pixel.                                                 |
 | Retonra: Nada.                                                          |
\*-------------------------------------------------------------------------*/
void setSVGAHI (int x, int y, int cor)
{
  unsigned long offset;

  static char bank,
	      bank_ant = 0;

  char far *tela = MK_FP (0xA000, 0x0000); /* Endereco da placa de video. */

  offset = (unsigned long) y * 1024 + x;
  bank = offset >> 16;
  offset = offset & 0xFFFF;

  if (bank != bank_ant)  {  /* Setar o banco de memoria de video. */
    asm cli
    asm Push   AX
    asm Push   DX
    asm Mov    AH, bank
    asm Xor    AH, 2
    asm Mov    DX, 0x3c4
    asm Mov    AL, 0Eh
    asm Out    DX, AX
    asm Pop DX
    asm Pop AX
    asm sti
    bank_ant = bank;
  }

  tela [offset] = cor;
}

void muda_64_tons_cinza ()
{
  int i;

  for (i = 0; i < 64; i++) {
    palette [i] [0] = i;
    palette [i] [1] = i;
    palette [i] [2] = i;
  }
  for (i = 64; i < 128; i++) {
    palette [i] [0] = i;
    palette [i] [1] = 0;
    palette [i] [2] = 0;
  }
  for (i = 128; i < 192; i++) {
    palette [i] [0] = 0;
    palette [i] [1] = i;
    palette [i] [2] = 0;
  }
  for (i = 192; i < 256; i++) {
    palette [i] [0] = 0;
    palette [i] [1] = 0;
    palette [i] [2] = i;
  }
  for (i = 0; i < 63; i++) {
    palette [i] [0] = i;
    palette [i] [1] = i;
    palette [i] [2] = i;
  }

  asm Push ES
  asm Push DX
  asm Push AX
  asm Push BX
  asm Push CX
  asm Push DX
  asm Mov AX, DS
  asm Mov ES, AX
  asm Lea DX, palette
  asm Mov BX, 0
  asm Mov CX, 256
  asm Mov AX, 0x1012
  asm Int 0x10
  asm Pop DX
  asm Pop CX
  asm Pop BX
  asm Pop AX
  asm Pop DX
  asm Pop ES

}

/*-------------------------------------------------------------------------*\
 | fim_modo_graf: Retorna ao modo vga texto (80 x 25 colunas texto).       |
 | Recebe: Nada.                                                           |
 | Retorna: Nada.                                                          |
\*-------------------------------------------------------------------------*/
void fim_modo_graf (void)
{
  union REGS reg_entr,
	     reg_saida;

  reg_entr.h.ah = 0;
  reg_entr.h.al = 3; /* Modo 80 linhas, 25 coluna modo texto. */
  int86 (0x10, &reg_entr, &reg_saida);
}
